
cd ../../Pop/scalable/apps

ln -s disk-usage-analyzer-symbolic.svg org.gnome.baobab-symbolic.svg
ln -s accessories-calculator-symbolic.svg org.gnome.Calculator-symbolic.svg
ln -s office-calendar-symbolic.svg org.gnome.Calendar-symbolic.svg
ln -s addressbook-symbolic.svg org.gnome.Contacts-symbolic.svg
ln -s disks-symbolic.svg org.gnome.DiskUtility-symbolic.svg
ln -s multimedia-photo-viewer-symbolic.svg org.gnome.eog-symbolic.svg
ln -s document-viewer-symbolic.svg org.gnome.Evince-symbolic.svg
ln -s system-file-manager-symbolic.svg org.gnome.Files-symbolic.svg
ln -s accessories-text-editor-symbolic.svg org.gnome.gedit-symbolic.svg
ln -s system-file-manager-symbolic.svg org.gnome.Nautilus-symbolic.svg
ln -s password-manager-symbolic.svg org.gnome.seahorse.Application-symbolic.svg
ln -s multimedia-video-player-symbolic.svg org.gnome.Totem-symbolic.svg
ln -s weather-symbolic.svg org.gnome.weather-symbolic.svg
ln -s weather-symbolic.svg org.gnome.Weather-symbolic.svg
ln -s help-contents-symbolic.svg org.gnome.Yelp-symbolic.svg
ln -s gnome-power-manager-symbolic.svg org.gnome.PowerStats-symbolic.svg
ln -s application-font-symbolic.svg org.gnome.font-viewer-symbolic.svg
ln -s utilities-terminal-symbolic.svg org.gnome.Terminal-symbolic.svg
