#!/bin/bash

./render-bitmaps.py distributor-logo-antergos
./render-bitmaps.py distributor-logo-manjaro
./render-bitmaps.py distributor-logo-arch-linux
./render-bitmaps.py distributor-logo-opensuse
./render-bitmaps.py distributor-logo-debian
./render-bitmaps.py distributor-logo-pop-os
./render-bitmaps.py distributor-logo-elementaryos
./render-bitmaps.py distributor-logo-slackware
./render-bitmaps.py distributor-logo-fedora
./render-bitmaps.py distributor-logo-solus
./render-bitmaps.py distributor-logo-korora
./render-bitmaps.py distributor-logo-ubuntu-gnome
./render-bitmaps.py distributor-logo-kubuntu
./render-bitmaps.py distributor-logo- distributor-logo-ubuntu-mate
./render-bitmaps.py distributor-logo-linux-mint
./render-bitmaps.py distributor-logo-ubuntu
./render-bitmaps.py distributor-logo-lubuntu
./render-bitmaps.py distributor-logo- distributor-logo-xubuntu
./render-bitmaps.py distributor-logo-mageia
./render-bitmaps.py distributor-logo-popos
